/*  dvbcut
    Copyright (c) 2005 Sven Over <svenover@svenover.de>
 
    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
 
    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
 
    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

/* $Id: avframe.cpp 141 2009-01-09 17:53:48Z too-tired $ */

#include <qimage.h>
#include <cstdlib>
#include <cstdio>
#include "avframe.h"

#define IMAGE_ALIGN_VALUE 1 // no idea what an ideal value is.

#ifdef HAVE_LIB_SWSCALE
avframe::avframe() : tobefreed(0),w(0),h(0),dw(0),pix_fmt(),img_convert_ctx(0)
#else
avframe::avframe() : tobefreed(0),w(0),h(0),dw(0),pix_fmt()
#endif
  {
  f=av_frame_alloc();
  }

avframe::avframe(AVFrame *src, AVCodecContext *ctx) : f(0),tobefreed(0)
  {
  f=av_frame_alloc();
/*
int avpicture_get_size       (enum AVPixelFormat pix_fmt, int width, int height) 	
Deprecated:
    use av_image_get_buffer_size() instead:
int av_image_get_buffer_size (enum AVPixelFormat pix_fmt, int width, int height, int align)
https://www.ffmpeg.org/doxygen/3.0/group__lavu__picture.html#ga24a67963c3ae0054a2a4bab35930e694	

*/
  tobefreed=malloc(av_image_get_buffer_size(ctx->pix_fmt, ctx->width, ctx->height, IMAGE_ALIGN_VALUE));

/*
int avpicture_fill      (AVPicture * picture, const uint8_t * ptr
    , enum AVPixelFormat pix_fmt, int width, int height)
Deprecated:
    use av_image_fill_arrays() instead:
int av_image_fill_arrays(uint8_t * dst_data[4], int dst_linesize[4], const uint8_t * src
    ,	enum AVPixelFormat pix_fmt, int width, int height, int align) 	
*/
  av_image_fill_arrays(f->data, f->linesize,
                 (u_int8_t*)tobefreed,
                 ctx->pix_fmt, ctx->width, ctx->height, IMAGE_ALIGN_VALUE);

#if LIBAVCODEC_VERSION_INT >= AV_VERSION_INT(51, 0, 0)
/*
void av_picture_copy(AVPicture * dst, const AVPicture * src
    , enum AVPixelFormat pix_fmt, int	width, int height) 	
Deprecated: av_image_copy() instead:
void av_image_copy 	(uint8_t * dst_data[4], int dst_linesizes[4]
    , const uint8_t * src_data[4], const int src_linesizes[4]
		, enum AVPixelFormat pix_fmt, int width, int height)
*/
  av_image_copy(f->data,  f->linesize, (const uint8_t **)src->data, src->linesize,
                  ctx->pix_fmt, ctx->width, ctx->height);
#else
  img_copy((AVPicture *)f, (const AVPicture *) src,
           ctx->pix_fmt, ctx->width, ctx->height);
#endif

  f->pict_type              = src->pict_type;
  f->quality                = src->quality;
  f->coded_picture_number   = src->coded_picture_number;
  f->display_picture_number = src->display_picture_number;
  f->pts                    = src->pts;
  f->interlaced_frame       = src->interlaced_frame;
  f->top_field_first        = src->top_field_first;
  f->repeat_pict            = src->repeat_pict;
  f->quality                = src->quality;

  w=ctx->width;
  h=ctx->height;
  pix_fmt=ctx->pix_fmt;
  dw=w*ctx->sample_aspect_ratio.num/ctx->sample_aspect_ratio.den;
#ifdef HAVE_LIB_SWSCALE
  img_convert_ctx=sws_getContext(w, h, pix_fmt, 
                                 w, h, AV_PIX_FMT_BGR24, SWS_BICUBIC, 
                                 NULL, NULL, NULL);
#endif
  }

avframe::~avframe()
  {
  if (tobefreed)
    free(tobefreed);
  if (f)
    av_free(f);
#ifdef HAVE_LIB_SWSCALE
  if (img_convert_ctx)
    sws_freeContext(img_convert_ctx);
#endif
  }

QImage avframe::getqimage(bool scaled, double viewscalefactor)
  {
#ifdef HAVE_LIB_SWSCALE
  if (w<=0 || h<=0 || img_convert_ctx==NULL)
#else
  if (w<=0 || h<=0)
#endif
    return QImage();

  uint8_t *rgbbuffer=(uint8_t*)malloc(av_image_get_buffer_size(AV_PIX_FMT_RGB24, w, h, IMAGE_ALIGN_VALUE)+64);
  int headerlen=sprintf((char *) rgbbuffer, "P6\n%d %d\n255\n", w, h);

  AVFrame *avframergb=av_frame_alloc();

  av_image_fill_arrays(avframergb->data, avframergb->linesize,
                 rgbbuffer+headerlen,
                 AV_PIX_FMT_RGB24, w, h, IMAGE_ALIGN_VALUE);

#ifdef HAVE_LIB_SWSCALE
  sws_scale(img_convert_ctx, f->data, f->linesize, 0, h, 
              avframergb->data, avframergb->linesize);
#else
  img_convert((AVPicture *)avframergb, AV_PIX_FMT_RGB24, (AVPicture*)f, pix_fmt, w, h);
#endif

  QImage im;
  im.loadFromData(rgbbuffer, headerlen+w*h*3, "PPM");

#ifdef HAVE_LIB_SWSCALE
  im = im.swapRGB();
#endif

  if ((scaled && w!=dw)||(viewscalefactor!=1.0)) {
#ifdef SMOOTHSCALE
    im = im.smoothScale(int((scaled?dw:w)/viewscalefactor+0.5), int(h/viewscalefactor+0.5));
#else
    im = im.scale(int((scaled?dw:w)/viewscalefactor+0.5), int(h/viewscalefactor+0.5));
#endif
    }

  free(rgbbuffer);
  av_free(avframergb);
  return (im);
  }
